use crate::prelude::*;

use std::{collections::HashMap, fmt::Debug, path::Path};

use config::{builder::DefaultState, Config as CConfig, ConfigBuilder};
use serde::{Deserialize, Serialize};

const CURRENT_VERSION: u64 = 1;

#[derive(Serialize, Deserialize, Debug, Clone)]
pub struct Gitlab {
    pub url: String,
}

#[derive(Serialize, Deserialize, Debug, Clone)]
pub struct Jira {
    pub url: String,
}

#[derive(Serialize, Deserialize, Debug, Clone)]
pub struct Config {
    pub version: u64,
    pub gitlab: Gitlab,
    pub jira: Jira,
}

impl Config {
    pub fn load<P>(config_file_path: Option<P>) -> Result<Self>
    where
        P: AsRef<Path>,
    {
        let c = Self::builder(config_file_path)?.build()?;
        Ok(c.try_deserialize()?)
    }

    pub fn builder<P>(config_file_path: Option<P>) -> Result<ConfigBuilder<DefaultState>>
    where
        P: AsRef<Path>,
    {
        let config_file_path: Box<dyn AsRef<Path>> = config_file_path
            .as_ref()
            .map(|s| Ok::<Box<dyn AsRef<Path>>, anyhow::Error>(Box::new(s.as_ref())))
            .unwrap_or_else(|| {
                Ok(Box::new(
                    xdg::BaseDirectories::new()?.get_config_file("taskr/config.toml"),
                ))
            })?;

        Ok(Self::default_builder()
            .map_err(anyhow::Error::from)?
            .add_source(config::File::from((*config_file_path).as_ref()).required(false))
            .add_source(config::Environment::with_prefix("taskr").separator("__")))
    }

    pub fn default_builder() -> Result<ConfigBuilder<DefaultState>> {
        Ok(CConfig::builder()
            .set_default("version", CURRENT_VERSION)?
            .set_default("jira", Option::<HashMap<String, config::Value>>::None)?
            .set_default("gitlab", Option::<HashMap<String, config::Value>>::None)?)
    }

    #[allow(dead_code)]
    pub fn defaults() -> Result<Self> {
        let c = Self::default_builder()?.build()?;
        Ok(c.try_deserialize()?)
    }
}

#[cfg(test)]
mod tests {
    use crate::config::*;

    #[test]
    fn defaults_deserialize_to_valid_config() -> Result<()> {
        println!("Defaults: {:?}", Config::defaults()?);
        Ok(())
    }
}
