use crate::prelude::*;

use crossterm::{
    event::{self, KeyCode, KeyEventKind, KeyModifiers},
    terminal::{disable_raw_mode, enable_raw_mode, EnterAlternateScreen, LeaveAlternateScreen},
    ExecutableCommand,
};
use futures::stream::StreamExt;
use ratatui::{
    prelude::{CrosstermBackend, Stylize, Terminal},
    widgets::Paragraph,
};
use signal_hook::consts::*;
use signal_hook_tokio::Signals;
use std::io::stdout;

pub struct Tui {
    tasks: SharedTasks,
}

impl Tui {
    pub fn new(tasks: SharedTasks) -> Self {
        Self { tasks }
    }

    pub async fn run(&self) -> Result<()> {
        self.tui().await
    }

    async fn tui(&self) -> Result<()> {
        stdout().execute(EnterAlternateScreen)?;
        enable_raw_mode()?;
        let mut terminal = Terminal::new(CrosstermBackend::new(stdout()))?;
        terminal.clear()?;
        let mut signals = Signals::new(&[SIGHUP, SIGTERM, SIGINT, SIGQUIT])?;
        let signalled: Arc<Mutex<bool>> = Arc::new(Mutex::new(false));
        let handle = signals.handle();

        let signaler = Arc::clone(&signalled);
        tokio::spawn(async move {
            // await a signal - once we receive one mark the boolean as true to
            // quit the app
            signals.next().await;
            let mut signalled = signaler.lock().await;
            *signalled = true;
        });
        // TODO: enable isig to re-allow ctrl-z?

        loop {
            terminal.draw(|frame| {
                let area = frame.size();
                frame.render_widget(
                    Paragraph::new("Hello Ratatui! (press 'q' to quit)").white(),
                    area,
                );
            })?;
            if event::poll(std::time::Duration::from_millis(10))? {
                if let event::Event::Key(key) = event::read()? {
                    if key.kind == KeyEventKind::Press
                        && key.code == KeyCode::Char('c')
                        && key.modifiers == KeyModifiers::CONTROL
                    {
                        break;
                    }
                    if key.kind == KeyEventKind::Press && key.code == KeyCode::Char('q') {
                        break;
                    }
                }
            }
            if *(*signalled).lock().await {
                break;
            }
        }

        handle.close();
        stdout().execute(LeaveAlternateScreen)?;
        disable_raw_mode()?;
        Ok(())
    }
}
